
#include <iostream>
#include <string>
#include <vector>

#include "tcbr/Entity.h"
#include "tcbr/KnowledgeModel.h"
#include "tcbr/Relation.h"
#include "tcbr/CaseBase.h"
#include "tcbr/Case.h"
#include "tcbr/SimilarityMeasure.h"
#include "tcbr/CaseRetriever.h"

#include "util/CaseBaseGenerator.h"

#include "dlearn/FeatureRanker.h"
#include "dlearn/CcbrEngine.h"
#include "dlearn/Agent.h"

using namespace std;
using namespace TCBR_Representation;
using namespace CCBR_Module;

double averageLength(vector<unsigned int> lengths) {
	double avg = 0.0;

	double sum = 0.0;
	for(unsigned int i=0; i<lengths.size(); i++)
		sum += lengths[i];

	avg = sum / lengths.size();
	return 	avg;
}

int main (int argc, char* argv[]){
	time_t timeStart = time(NULL);

	// Setup variables
	string solutionFeature = "db_primdiagnosis";
	string datafile = "W:\\dist\\tabseparertDatasett.dat";


	//string solutionFeature = "class";
	//string datafile = "W:\\dist\\soybean-large.dat";
	//string datafile = "W:\\dist\\lung-cancer.dat";


	double problemDescriptionSize = 0.10; // part of the features
	double applicationSimThreshold = 0.7;
	double dialogueSimThreshold = 0.5;
	int numberOfCases = 700;
	unsigned int cycles = 2;
	int minimumSolutionRank = 1;

	int startPos = -1;
	int minimumSolutionCount = 1;
	bool isPatc = true;

    int option = 1;
    // decode arguments
    while((option < argc) && (argv[option][0]=='-')) {
        string sw = argv[option];
 		if (sw=="-sof") {
            solutionFeature = argv[++option];
        } else if (sw=="-dat") {
            datafile = argv[++option];
        } else if (sw=="-npat") {
        	isPatc = false;
        } else if (sw=="-pos") {
            startPos = atoi(argv[++option]);
        } else if (sw=="-noc") {
            numberOfCases = atoi(argv[++option]);
        } else if (sw=="-msc") {
            minimumSolutionCount = atoi(argv[++option]);
        } else if (sw=="-msr") {
            minimumSolutionRank = atoi(argv[++option]);
        } else if (sw=="-cyc") {
            cycles = atoi(argv[++option]);
        } else if (sw=="-pds") {
            problemDescriptionSize = atof(argv[++option]);
        } else if (sw=="-ast") {
            applicationSimThreshold = atof(argv[++option]);
        } else if (sw=="-dst") {
            dialogueSimThreshold = atof(argv[++option]);
        }
        option++;
    }


	// Variables containing experiment results:
	unsigned int applicationCases;			// The number of cases in the application case base
	unsigned int applicationFeatures;		// The number of feature types in the app case base (not including the solution)
	unsigned int applicationSolutions;		// The number of distinct solutions in the case base

	vector<vector<unsigned int> > dlnl;		// Each cycle have a vector of dialogue lengths, no learning
	vector<vector<unsigned int> > dll;		// Each cycle have a vector of dialogue lengths, learning

	vector<unsigned int> obtainedCases;		// The number of new dialogue cases retained in the cycle


	// Create casebase and insert cases
	CaseBase* caseBase = new CaseBase("ACBase"); // Case base for application cases
	CaseBaseGenerator* cbg = new CaseBaseGenerator(datafile, isPatc);

	if(startPos < 0)
		cbg->insertRandom(numberOfCases, caseBase);
	else
		cbg->insertSelection(startPos, startPos+numberOfCases, caseBase);

	/*
	// Analyzing the data with 1-NN:
	QueryBiasedSymbolicComparator* sc = new QueryBiasedSymbolicComparator();
	SimilarityMeasureCaseRetriever* crsc = new SimilarityMeasureCaseRetriever(caseBase, sc, 0.0);

	set<Case*> cases = caseBase->getActiveCases();

	for (set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
		Case* c = *it;
		string n = c->getName();

		string baseSolution = c->getFeature(solutionFeature)->getValue()->getTextValue();

		vector<pair<Case*, double> > result = crsc->retrieveSimilarCases(c);

		// Most similar
		string similarSolution = result[0].first->getFeature(solutionFeature)->getValue()->getTextValue();

		// Most similar correct
		bool found = false;
		for(vector<pair<Case*, double > >::const_iterator iter = result.begin(); iter != result.end(); ++iter) {
			Case* c2 = (*iter).first;
			string matchSolution = c2->getFeature(solutionFeature)->getValue()->getTextValue();
			if(baseSolution == matchSolution){
				cout << c->getName() << "\t" << baseSolution << "\t" << similarSolution << "\t" << result[0].second << "\t" << (*iter).second << endl;
				found = true;
				break;
			}
		}
		if(!found)
			cout << c->getName() << "\t" << baseSolution << "\t" << similarSolution << "\t" << result[0].second << "\t" << 0 << endl;
	}
	*/


	// Remove cases where only one case has the same solution
	set<Case*> casesset = caseBase->getCases();
	map<string, vector<Case*> > solutionCount;
	for(set<Case*>::const_iterator it = casesset.begin(); it != casesset.end(); ++it) {
		Case* c = *it;
		string s = c->getFeature(solutionFeature)->getValue()->getTextValue();

		map<string, vector<Case*> >::iterator mapit = solutionCount.find(s);
		if(mapit != solutionCount.end()) {
			(*mapit).second.push_back(c);
		} else {
			vector<Case*> cs;
			cs.push_back(c);
			solutionCount.insert(make_pair(s, cs));
		}
	}

	for (map<string, vector<Case*> >::iterator it = solutionCount.begin(); it != solutionCount.end(); ++it) {
		vector<Case*> cs = (*it).second;
		//cout << (*it).first << " " << cs.size() << endl;
		if(cs.size()< minimumSolutionCount){
			for(unsigned int i=0; i<cs.size(); i++)
				delete cs[i];
		}
	}

	// Find case base characteristics
	applicationCases = caseBase->getActiveCases().size();
	applicationFeatures = caseBase->getFeatureTypes().size();
	applicationSolutions = caseBase->getFeatureValues(solutionFeature).size();

	cout << "MAIN-> Case base characteristics:" << endl;
	cout << "MAIN->    Cases:    \t" << applicationCases << endl;
	cout << "MAIN->    Features: \t" << applicationFeatures << endl;
	cout << "MAIN->    Solutions:\t" << applicationSolutions << endl;

	// Global weighting of features
	FeatureRanker* ranker = new FeatureRanker(caseBase, solutionFeature);
	cout << endl << "MAIN-> Global feature ranking started..." << endl;
	ranker->rankFeatures();

	// Create the ccbr engine
	CcbrEngine* ccbr = new CcbrEngine(caseBase, solutionFeature,  applicationSimThreshold, dialogueSimThreshold);

	// Create a agent which function as a user talking with the ccbrengine
	Agent* agent = new Agent(ccbr, solutionFeature, minimumSolutionRank);

	set<Case*> cases = caseBase->getActiveCases();

	// Without Dialogue Learning
	ccbr->disableDialogueLearning();
	for(unsigned int cyc = 0; cyc < cycles; cyc++) {
		vector<unsigned int> nl;
		for(set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
			Case* testcase = *it;
			testcase->deactivate(); // leave this one out

			cout << endl << "MAIN-> Starting simulation. (Case: " << testcase->getName() << ") " << (difftime(time(NULL), timeStart)/60) << "min" << endl << endl;
			nl.push_back(agent->startSimulation(testcase, problemDescriptionSize));
			cout << endl << "MAIN-> Simulation complete. " << (difftime(time(NULL), timeStart)/60) << "min" << endl << endl;

			testcase->activate(); // reactivate the case
		}
		dlnl.push_back(nl);
	}

	// With Dialogue Learning
	ccbr->enableDialogueLearning();
	for(unsigned int cyc = 0; cyc < cycles; cyc++) {
		vector<unsigned int> dl;
		for(set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
			Case* testcase = *it;
			testcase->deactivate(); // leave this one out

			cout << endl << "MAIN-> Starting simulation. (Case: " << testcase->getName() << ") " << (difftime(time(NULL), timeStart)/60) << "min" << endl << endl;
			dl.push_back(agent->startSimulation(testcase, problemDescriptionSize));
			cout << endl << "MAIN-> Simulation complete. " << (difftime(time(NULL), timeStart)/60) << "min" << endl << endl;

			testcase->activate(); // reactivate the case
		}
		dll.push_back(dl);
		obtainedCases.push_back(ccbr->getNumberOfStoredDialogues());
	}

	cout << endl << "MAIN-> Results for this case base: " << endl;
	cout << "MAIN-> Total time:\t" << (difftime(time(NULL), timeStart)/60) << "min" << endl << endl;

	cout << "MAIN-> Total cases:\t" << applicationCases << endl;
	cout << "MAIN-> Features:\t" << applicationFeatures << endl;
	cout << "MAIN-> Solutions:\t" << applicationSolutions << endl;
	for(unsigned int i=0; i<cycles; i++) {
		cout << "MAIN-> Cycle " << i << endl;
		cout << "MAIN->   Dialogue length, no learning:\t" << averageLength(dlnl[i]) << endl;
		cout << "MAIN->   Dialogue length, learning:\t" << averageLength(dll[i]) << endl;
		cout << "MAIN->   Stored dialogue cases:\t" << obtainedCases[i] << endl;
	}

	char theEnd;
	cout << "Press any key to terminate experiment.";
	cin >> theEnd;

	return 0;
}

