#include "Agent.h"
#include <time.h>

namespace CCBR_Module{

	Agent::Agent(CcbrEngine* ccbrEngine, string solutionFeature, int minRank):ccbrEngine_(ccbrEngine), solutionFeature_(solutionFeature), minRank_(minRank){
		srand((unsigned)time(NULL));
	}

	Agent::~Agent(){}

	int Agent::startSimulation(Case* problem, double problemDescriptionSize){
		dialogueLength_ = 0;
		problem_ = problem;
		problemSolution_ = problem_->getFeature(solutionFeature_)->getValue()->getTextValue();

		vector<Feature*> features = problem_->getFeatures();
		vector<pair<string, string> > problemDescription;

		string type, value;
		unsigned int numberOfFeatures = features.size() * problemDescriptionSize;
		cout << "AGENT-> Selecting " << numberOfFeatures << " features for problemdescription" << endl;
		for (unsigned int i = 0; i < numberOfFeatures; ) {
			int pos = (rand() % (features.size()-i));
			type = features[pos]->getType()->getName();
			if(type != solutionFeature_) {
				//cout << "AGENT-> Feature: " << type << endl;
				value = features[pos]->getValue()->getTextValue();
				features[pos] = features[features.size()-1-i];
				problemDescription.push_back(make_pair(type, value));
				dialogueLength_++;
				i++;
			}
		}

		bool success = ccbrEngine_->query(problemDescription, this);

		cout << "AGENT-> Simulation: " << success << " Length: " << dialogueLength_ << endl;

		return dialogueLength_;
	}

	pair<string, string> Agent::askQuestions(vector<pair<string, double> >* questions) {
		assert(questions);
		dialogueLength_++;
		pair<string, string> answer;

		// Prints the questions presented to the agent
		cout << "AGENT-> Answers best ranked question: " << questions->at(0).first << " (" << questions->at(0).second << ")"<< endl;
/*
		for (unsigned int i = 0; i < questions->size() && i<5; i++) {
			cout << questions->at(i).first << " (" << questions->at(i).second << ")" << endl;
		}
*/
		//Simply selects the best ranked question that we can answer
		Feature* answerFeature = NULL;
		for (unsigned int i = 0; i < questions->size() && answerFeature == NULL; i++) {
			if(problem_->hasFeature(questions->at(i).first)) {
				answerFeature = problem_->getFeature(questions->at(i).first);
			}
		}
		if(answerFeature == NULL) {
			cout << "Case size " << problem_->getFeatures().size() << endl;
			return make_pair("","");
		} else {
			return make_pair(answerFeature->getType()->getName(), answerFeature->getValue()->getTextValue());
		}
	}

	Case* Agent::askSolutions(vector<pair<Case*, double> >* solutions) {
		// Prints the solutions presented to the agent
		cout << endl;
		cout << "AGENT-> Solution display " << dialogueLength_ << "(Correct: " << problemSolution_ << ")" << endl;

		int mr = solutions->size();
		if(minRank_ != 0)
			mr = minRank_;

		for (unsigned int i = 0; i < solutions->size() && i < minRank_; i++) {
			cout << solutions->at(i).first->getName() << " (" << solutions->at(i).second << ") Solution: " << solutions->at(i).first->getFeature(solutionFeature_)->getValue()->getTextValue() << endl;
		}
		cout << endl;

		// Check whether the best ranked solution is the correct, else continues ccbr cycle

		for (unsigned int i = 0; i < minRank_; i++) {
			if(solutions->at(i).first->getFeature(solutionFeature_)->getValue()->getTextValue() == problemSolution_) {
				cout << "AGENT-> Solution correct. " << solutions->at(i).first->getName() << endl;
				return solutions->at(i).first;
			}
		}
		return NULL;
	}

}

