#include "FeatureRanker.h"

namespace CCBR_Module{

	FeatureRanker::FeatureRanker(CaseBase* caseBase, string solutionFeature):caseBase_(caseBase),solutionFeature_(solutionFeature){
		// add all features with weight 1 (as default)
		set<Entity*> featureTypes = caseBase_->getFeatureTypes();
		for (set<Entity*>::iterator it = featureTypes.begin(); it != featureTypes.end(); ++it) {
			if((*it)->getName() != solutionFeature_)
				weights_.insert(make_pair((*it)->getName(), 1.0));
		}
	}

	void FeatureRanker::rankFeatures() {
		QueryBiasedSymbolicComparator* sc = new QueryBiasedSymbolicComparator();
		SimilarityMeasureCaseRetriever* crsc = new SimilarityMeasureCaseRetriever(caseBase_, sc, 0.0);

		set<Case*> cases = caseBase_->getActiveCases();

		for (set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
			Case* c = *it;
			cout << "FR-> Processing " << c->getName() << endl;
			processCase(c, crsc, 0.5);
		}

		// Normalize weights to sumarize to 1.0
		this->normalizeWeights();

		// Set the weights
		for (map<string, double>::iterator it = weights_.begin(); it != weights_.end(); ++it) {
			caseBase_->setWeight((*it).first, (*it).second);
			//cout << (*it).first << ": " << (*it).second << endl;
		}
	}

	FeatureRanker::~FeatureRanker(){}

	void FeatureRanker::normalizeWeights() {
		cout << "FR-> Normalizing weights." << endl;
		double lowest = ((pair<string, double>)*weights_.begin()).second;

		for (map<string, double>::iterator it = weights_.begin(); it != weights_.end(); ++it) {
			if((*it).second<lowest)
				lowest = (*it).second;
		}

		double sum = 0;
		for (map<string, double>::iterator it = weights_.begin(); it != weights_.end(); ++it) {
			(*it).second = (*it).second - lowest;
			sum += (*it).second;
		}

		for (map<string, double>::iterator it = weights_.begin(); it != weights_.end(); ++it) {
			(*it).second = (*it).second / sum;
		}
	}

	void FeatureRanker::processCase(Case* c, SimilarityMeasureCaseRetriever* crsc, double deltaWeight) {

		Case* mostSimilarCase = crsc->retrieveSimilarCases(c)[0].first;
		string matchSolution = mostSimilarCase->getFeature(solutionFeature_)->getValue()->getTextValue();
		string caseSolution = c->getFeature(solutionFeature_)->getValue()->getTextValue();

		if(matchSolution == caseSolution) {
			//inc
			updateMatchingFeatures(c, mostSimilarCase, deltaWeight);
		} else {
			//dec
			updateMatchingFeatures(c, mostSimilarCase, (-1 * deltaWeight));
		}
	}

	void FeatureRanker::updateMatchingFeatures(Case* c1, Case* c2, double deltaWeight) {

		set<Entity*> featureTypes = caseBase_->getFeatureTypes();

		for (set<Entity*>::iterator it = featureTypes.begin(); it != featureTypes.end(); ++it) {
			Entity* ft = *it;
			string name = ft->getName();
			if(name != solutionFeature_) {
				if(c1->hasFeature(name) && c2->hasFeature(name)) {
					if(c1->getFeature(name)->getValue()->getTextValue() == c2->getFeature(name)->getValue()->getTextValue()) {
						map<string, double>::iterator mapit = weights_.find(name);
						//if(mapit != weights_.end()) {
							(*mapit).second = (*mapit).second+deltaWeight;
						//} else {
						//	weights_.insert(make_pair(name, deltaWeight));
						//}
					}
				}
			}
		}
	}

}
