#include "InformationGain.h"

namespace CCBR_Module{

	InformationGain::InformationGain(){}

	InformationGain::~InformationGain(){}

	double InformationGain::calculateEntropy(string cFeature, vector<pair<Case*, double> > sim, CaseBase* cb){
		double entropy = 0.0;
		set<Entity*> classificationValues = cb->getFeatureValues(cFeature);
		for (set<Entity*>::const_iterator it = classificationValues.begin(); it != classificationValues.end(); ++it) {
			Entity* e = *it;
			double n = 0.0; //counts the number cases categorized by e
			//cout << "Entropy: REGNER UT FOR " << e->getName() << endl;
			for (unsigned int i = 0; i < sim.size(); i++) {	//telle antall like i settet med case
				Case* c = sim[i].first;
				if(c->hasFeature(cFeature)){ //maybe unnecessary?
					Feature* f = c->getFeature(cFeature);
					Value* v = f->getValue();
					if((e->getName()).compare(v->getTextValue())==0) n++;
				}
			}
			if(n!=0)entropy -= (n/sim.size())*log2(n/sim.size());
			//cout << "Antall " << n <<"  entropy " << entropy << endl;
		}
		//cout<<"The entropy is " << entropy << endl;
		return entropy;

	}

	double InformationGain::calculateReduction(string feature, string cFeature, vector<pair<Case*, double> > sim, CaseBase* cb){
		// for alle mulige verdier av en feature. maa finne antall mulige verdier for en feature
		double reduction = 0.0;
		set<Entity*> featureValues = cb->getFeatureValues(feature);
		for (set<Entity*>::const_iterator it = featureValues.begin(); it != featureValues.end(); ++it) {
			Entity* e = *it;
			double n = 0.0; //number of cases with value
			vector<pair<Case*, double> > caseSet; //lage et sett med case for hver verdi
			//cout << "REGNER UT FOR " << e->getName()<< endl;
			for (unsigned int i = 0; i < sim.size(); i++) {
				Case* c = sim[i].first;
				if(c->hasFeature(feature)){
					Feature* f = c->getFeature(feature);
					Value* v = f->getValue();
					if((e->getName()).compare(v->getTextValue())==0){
						 n++;
						 caseSet.push_back(make_pair(c, sim[i].second));
					}
				}
			}
			//cout<<"Antallet er "<<n<<endl;
			reduction += (n/sim.size())*calculateEntropy(cFeature,caseSet,cb);
			//cout<<reduction<<endl;
		}
		//cout<<"The reduction is " <<reduction<<endl;
		return reduction;
	}

	double InformationGain::calculateInformationGain(double entropy, string feature, string cFeature, vector<pair<Case*, double> > sim, CaseBase* cb){
		double informationGain = entropy - calculateReduction(feature,cFeature,sim, cb);
		//cout<<"The information gain for feature " << feature <<" is "<< informationGain <<endl;
		return informationGain;
	}

	vector<pair<Entity*, double> > InformationGain::rankFeatures(Case* query, string classification, vector<pair<Case*, double> > caseSimilarities,CaseBase* cb){
		vector<pair<Entity*, double> > featureRanking;

		double entropy = calculateEntropy(classification,caseSimilarities,cb);

		set<Entity*> featureTypes = cb->getFeatureTypes(); //dette er settet med features som en mulig skal sporre om
		for (set<Entity*>::const_iterator it = featureTypes.begin(); it != featureTypes.end(); ++it) {
			Entity* e = *it;
			if(!query->hasFeature(e->getName())){ //checks if the query has a value for this feature
				if(classification.compare(e->getName())!=0){
					//ranking metod: Information Gain
					double informationGain = calculateInformationGain(entropy,e->getName(),classification,caseSimilarities, cb);
					featureRanking.push_back(make_pair(e,informationGain));
				}
			}
		}

		sort(featureRanking.begin(), featureRanking.end(),featurepair());
		/*
		cout << "Number of features is " << featureRanking.size()<<endl;
		for (unsigned int i = 1; i < featureRanking.size() & i < 10; i++) {
			cout << featureRanking[i].first->getName() << " (" << featureRanking[i].second << ") " <<endl;
		}
		*/
		return featureRanking;
	}
}
