#include "CaseRetriever.h"

#include "CaseBase.h"
#include "SimilarityMeasure.h"

#include <memory>
#include <cassert>
#include <map>

using namespace std;

namespace TCBR_Representation{

	CaseRetriever::CaseRetriever(CaseBase* caseBase): caseBase_(caseBase) {}

	SimilarityMeasureCaseRetriever::SimilarityMeasureCaseRetriever(CaseBase* caseBase, SimilarityMeasure* similarityMeasure, double threshold): CaseRetriever(caseBase), similarityMeasure_(similarityMeasure), threshold_(threshold) {}

	DialogueCaseRetriever::DialogueCaseRetriever(CaseBase* dialogueCaseBase, CaseBase* appCaseBase, DialogueCaseComparator* similarityMeasure, double threshold): SimilarityMeasureCaseRetriever(dialogueCaseBase, similarityMeasure, threshold), appCaseBase_(appCaseBase) {}

	struct gtcasepair
	{
		bool operator()(const pair<Case*, double>& a, const pair<Case*, double>& b) const {
			return a.second > b.second;
		}
	};

	// Returns the cases in the case base sorted by similarity to the input case
	vector<pair<Case*, double> > SimilarityMeasureCaseRetriever::retrieveSimilarCases(Case* inputCase)
	{
		vector<pair<Case*, double> > result;

		set<Case*> cases(caseBase_->getActiveCases()); // CHANGED: Now using only active cases

		for (set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
			if (*it != inputCase) {
				double similarity = similarityMeasure_->computeCaseSimilarity(inputCase, *it);
				if(similarity >= threshold_) {  // CHANGED: Added trehold check
					result.push_back(make_pair(*it, similarity));
				}
			}
		}

		sort(result.begin(), result.end(), gtcasepair());

		return result;
	}

	// Returns the cases in the case base sorted by similarity to the input case
	vector<pair<Case*, double> > DialogueCaseRetriever::retrieveSimilarCases(Case* inputCase, Case* appInputCase) {
		vector<pair<Case*, double> > result;

		set<Case*> cases(caseBase_->getActiveCases()); // CHANGED: Now using only active cases

		for (set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
			if (*it != inputCase) {

				Case* stored = *it;
				Feature* storedFeature = stored->getFeature("ds_solution");
				assert(storedFeature);
				Value* storedValue = storedFeature->getValue();
				assert(storedValue);
				string s = storedValue->getTextValue();

				Case* appSolution = dynamic_cast<Case*>(appCaseBase_->getEntity(s));

				double similarity = dynamic_cast<DialogueCaseComparator*>(similarityMeasure_)->computeCaseSimilarity(inputCase, stored, appInputCase, appSolution);

				if(similarity >= threshold_) {  // CHANGED: Added threhold check
					result.push_back(make_pair(*it, similarity));
				}
			}
		}

		sort(result.begin(), result.end(), gtcasepair());

		return result;
	}

}
