#include "ResultSet.h"

using namespace std;

namespace TCBR_Representation
{

ResultSet::ResultSet()
{
	cases_= new std::vector<TCBR_Representation::Case*>();
	similarityMap_ = new std::map<std::string, float>();
	highestSimilarityDegree_ = 0;
	lowestDegreeOfSimilarity_ = 0;
}

ResultSet::~ResultSet()
{
}

void ResultSet::addCase(Case* c, float degreeOfSimilarity)
{
	if (degreeOfSimilarity > highestSimilarityDegree_)
	{
		highestSimilarityDegree_ = degreeOfSimilarity;
	}

	if (degreeOfSimilarity < lowestDegreeOfSimilarity_)
	{
		lowestDegreeOfSimilarity_ = degreeOfSimilarity;
	}
	cases_->push_back(c);

	pair<map<std::string, float>::iterator, bool> ret;

	ret = similarityMap_->insert(make_pair(c->getName(), degreeOfSimilarity));
	if (ret.second)
	{
		cout << "Case: " << c->getName() << " inserted into ResultSet" << endl;
	}
	else
	{
		cout << "Failed to insert case: " << c->getName() << " into ResultSet" << endl;
	}
}

bool ResultSet::hasCase(string name)
{
	bool hasCase = false;

	unsigned int i;
	for (i = 0; i < cases_->size(); i++)
	{
		if (cases_->at(i)->getName().compare(name)==0)
		{
			hasCase = true;
			break;
		}
	}
	return hasCase;
}

vector<Case*>* ResultSet::getCases()
{
	return cases_;
}

//Returns -100 if name has noe similarity degree
float ResultSet::getSimilarityDegree(std::string name)
{
	float returnValue = -100;
	map<string, float>::iterator it = similarityMap_->find(name);
	if (it!=similarityMap_->end())
	{
		returnValue = it->second;
	}
	return returnValue;
}

float ResultSet::getHighestSimilarityDegree()
{
	return highestSimilarityDegree_;
}

float ResultSet::getLowestSimilarityDegree()
{
	return lowestDegreeOfSimilarity_;
}

void ResultSet::printContent()
{
	cout << "Cases in result set: " << endl;
	unsigned int i;
	for (i = 0; i < cases_->size(); i++)
	{
		cout << cases_->at(i)->getName() << endl;;
	}
}

}
