#include "Case.h"
#include "CaseBase.h"
#include "SimilarityMeasure.h"

#include <memory>
#include <cassert>
#include <cmath>
#include <iostream>

using namespace std;

namespace TCBR_Representation
{

double QueryBiasedSymbolicComparator::getSymbolicDifference(Case* from, Case* to, Entity* type)
{
	string name = type->getName();
	if (from->hasFeature(name) && to->hasFeature(name)) {
		Value* a = from->getFeature(name)->getValue();
		Value* b = to->getFeature(name)->getValue();

		assert(a);
		assert(b);

		// Symbolic value comparisons
		if (a->getTextValue() == b->getTextValue()) {
			return 0.0;
		} else {
			return 1.0;
		}
	} else {
		// Missing value counted the same as different symbolic value
		return 1.0;
	}
}

double QueryBiasedSymbolicComparator::computeCaseSimilarity(Case* from, Case* to)
{
	double distance = 0;

	const vector<Feature*>& fromFeatures(from->getFeatures());
	//const vector<Feature*>& toFeatures(to->getFeatures());

	assert(from->getKnowledgeModel() == to->getKnowledgeModel());
	CaseBase* cb = dynamic_cast<CaseBase*>(from->getKnowledgeModel());
	assert(cb);

	double sumWeights = 0; // NEW CODE
	// Query-biased comparison: only looking at the features in the from vector
	for (vector<Feature*>::const_iterator it = fromFeatures.begin(); it != fromFeatures.end(); ++it) {
		double weight = cb->getWeight((*it)->getType()->getName());
		Entity* type = (*it)->getType();
		double difference = this->getSymbolicDifference(from, to, type);
		distance += difference * weight;
		sumWeights += weight; // NEW CODE
	}

	distance = sqrt(distance / sumWeights); // NEW CODE

	double similarity = 1 / (distance + 1);
	return similarity;
}

double DialogueCaseComparator::computeCaseSimilarity(Case* query, Case* stored, Case* appQuery, Case* appSolution){
	double distance = 0;

	const vector<Feature*>& queryFeatures(query->getFeatures());

	assert(query->getKnowledgeModel() == stored->getKnowledgeModel());
	assert(appQuery->getKnowledgeModel() == appSolution->getKnowledgeModel());

	CaseBase* dcb = dynamic_cast<CaseBase*>(query->getKnowledgeModel());
	assert(dcb);
	CaseBase* acb = dynamic_cast<CaseBase*>(appQuery->getKnowledgeModel());
	assert(acb);

	double sumWeights = 0;
	// Query-biased comparison: only looking at the features in the from vector
	for (vector<Feature*>::const_iterator it = queryFeatures.begin(); it != queryFeatures.end(); ++it) {
		double weight = acb->getWeight((*it)->getType()->getName());
		Entity* type = (*it)->getType();

		// Difference computed between the appcases, as the values are stored there
		double difference = this->getSymbolicDifference(appQuery, appSolution, type);

		// Compute the weight concerning the appearance position of the feature
		double posw = this->getPositionWeight(query, stored, type);

		distance += weight * posw * difference;
		sumWeights += weight;
	}
	distance = sqrt(distance / sumWeights);

	double similarity = 1 / (distance + 1);
	return similarity;
}

double DialogueCaseComparator::getPositionWeight(Case* from, Case* to, Entity* type) {
	double posw = 0;
	if(to->hasFeature(type->getName())) {
		int posFrom = convertToInt(from->getFeature(type->getName())->getValue()->getTextValue());
		int posTo = convertToInt(to->getFeature(type->getName())->getValue()->getTextValue());
		int lengthFrom = from->getFeatures().size()-1; // -1 because the solution is included.
		int lengthTo = to->getFeatures().size()-1;

		posw = 0.5 + 0.5 * (1 - abs(posFrom-posTo) / max(lengthFrom, lengthTo));
	} else {
		posw = 0.5;
	}
	return posw;
}

}
