#include "CaseBaseGenerator.h"

CaseBaseGenerator::CaseBaseGenerator(string datafile){
	CaseBaseGenerator(datafile, true);
}

CaseBaseGenerator::CaseBaseGenerator(string datafile, bool patc){
	srand((unsigned)time(NULL));

	cases_.clear();
	string line;

	set<string> solutions;

	ifstream myfile(datafile.data());

	if (myfile.is_open()) {
		if(!patc) {
			// First line are feature names
			getline(myfile,line);
			line.erase(line.length()-1); // fjerne linjeskift
			vector<string> variables;

			char* pch;
			pch = strtok((char*)line.data(),"\t");
			while (pch != NULL){
				string s;
				s = s.append(pch);
				variables.push_back(s);
				pch = strtok (NULL, "\t");
			}

			vector<string> values;
			unsigned int casenumber = 0;
			getline(myfile,line);
			while(!myfile.eof()) {
				if(!values.empty())
					values.clear();

				line.erase(line.length()-1);

				pch = strtok((char*)line.data(),"\t");
				while (pch != NULL){
					string s;
					s = s.append(pch);
					values.push_back(s);
					pch = strtok (NULL, "\t");
				}

				vector<FEATURE> fs;
				FEATURE f;

				bool complete = true;
				unsigned int var = 0;
				while(var < values.size()) {
					if(values[var] != " ") {
						f.name = variables[var];
						f.value = values[var];
						fs.push_back(f);
					}
					var++;
				}
				CASE c;
				c.name = generateCaseId(++casenumber);
				c.features = fs;
				cases_.push_back(c);
				fs.clear();

				getline(myfile,line);
			}
		} else {
			// First line are feature names
			getline(myfile,line);
			line.erase(line.length()-1); // fjerne linjeskift
			vector<string> variables;
			unsigned int clincathunt = 0, clincateortc = 0, bgcalfright = 0;

			char* pch;
			pch = strtok((char*)line.data(),"\t");
			while (pch != NULL){
				string s;
				s = s.append(pch);
				variables.push_back(s);

				if(s.compare("db_clincathunt") == 0)
					clincathunt = variables.size()-1;
				else if(s.compare("db_clincateortc") == 0)
					clincateortc = variables.size()-1;
				else if(s.compare("db_bgcalfright") == 0)
					bgcalfright = variables.size()-1;

				pch = strtok (NULL, "\t");
			}

			vector<string> values;
			unsigned int casenumber = 0;
			getline(myfile,line);
			while(!myfile.eof()) {
				if(!values.empty())
					values.clear();

				line.erase(line.length()-1);
				//line.erase(line.length()-1); //?

				pch = strtok((char*)line.data(),"\t");
				while (pch != NULL){
					string s;
					s = s.append(pch);
					values.push_back(s);
					pch = strtok (NULL, "\t");
				}

				vector<FEATURE> fs;
				FEATURE f;

				bool complete = true;
				unsigned int var = 0;
				while(complete && var < clincathunt) {
					if(values[var] == " ") {
						complete = false;
					} else if(variables[var].compare("db_primdiagnosis") == 0) {
						if(values[var].size()>5) {
							complete = false;
	 					} else if(!isalpha(values[var][0])) {
	 						complete = false;
						} else {
	 						// Remove whitespaces
	 						string::size_type loc = values[var].find(" ", 0);
	 						while(loc != string::npos) {
	 							values[var].erase(loc,1);
	 							loc = values[var].find(" ", 0);
	 						}

	 						// Set all commas to "."
	 						loc = values[var].find(",", 0);
	 						while(loc != string::npos) {
	 							values[var][loc] = *(".");
	 							loc = values[var].find(",", 0);
	 						}

							// Set uppercase
							for(unsigned int i=0; i<values[var].length(); i++){
								values[var][i] = toupper(values[var][i]);
							}

							// More checks after changes
	 						if(!isdigit (values[var][1])) {
								complete = false;
	 						} else {

								// Insert comma when supposed to be there
								if(values[var].length() > 3) {
									if(isdigit(values[var][3]))
										values[var].insert(3,".");
								}

		 						solutions.insert(values[var]);
								f.name = variables[var];
								f.value = values[var];
								fs.push_back(f);
	 						}
						}
					} else {
						f.name = variables[var];
						f.value = values[var];
						fs.push_back(f);
					}
					var++;
				}

				if(complete) {
					if(values[clincathunt] == " " || values[clincateortc] == " ") {
						complete = false;
					} else {
						f.name = variables[clincathunt];
						f.value = values[clincathunt];
						fs.push_back(f);

						f.name = variables[clincateortc];
						f.value = values[clincateortc];
						fs.push_back(f);

						if(values[clincathunt] == "0" && values[clincateortc] == "0") {
							var = bgcalfright;
						}
						while(complete && var < values.size()) {
							if(values[var] == " ") {
								complete = false;
							} else {
								f.name = variables[var];
								f.value = values[var];
								fs.push_back(f);
							}
							var++;
						}
					}
				}

				if(complete) {
					CASE c;
					c.name = generateCaseId(++casenumber);
					c.features = fs;
					cases_.push_back(c);
					fs.clear();
				}

				getline(myfile,line);
			}

		}
		myfile.close();
	} else {
		cout << "CBG-> Unable to open file: " << datafile << endl;
	}

	cout << "CBG-> Stored " << cases_.size() << " cases from file: " << datafile << endl;

	// Initialize position set for use in random-methods
	for(unsigned int i=0; i<cases_.size(); i++)
		positions_.push_back(i);
}


CaseBaseGenerator::~CaseBaseGenerator(){}

void CaseBaseGenerator::insertRandom(unsigned int number, CaseBase* cb)
{
	insertRandom(number, 0, cases_.size(), cb);
}

// Insert random, but from a range..
void CaseBaseGenerator::insertRandom(unsigned int number, unsigned int from, unsigned int to, CaseBase* cb) {
	if((to-from) <= number) {
		cout << "CBG-> Number of cases equal to or larger than range." << endl;
		insertSelection(from,to,cb);
	} else {
		cout << "CBG-> Inserting " << number << " random cases from the range " << from << " to " << to << " in a dataset with " << cases_.size() << " cases, into casebase " << cb->getName() << endl;

		vector<int> positions(positions_);

		for(unsigned int n=0; n<number;n++)	{
			int pos = rand() % (to-from-1-n);
			pos += from;
			CASE c = cases_.at(positions[pos]);
			swap(positions[pos], positions[to-1-n]);

			Case* ac = new Case(c.name, cb);

			for (unsigned int i = 0; i < c.features.size(); i++) {
				FEATURE f = c.features.at(i);
				ac->addFeature(f.name, f.value);
			}
		}
	}
}

void CaseBaseGenerator::insertSelection(unsigned int from, unsigned int to, CaseBase* cb){
	cout << "CBG-> Inserting cases " << from << " to " << to << " from a dataset with " << cases_.size() << " cases, into casebase " << cb->getName() << endl;
	if(to>cases_.size()) {
		to = cases_.size();
	}
	for(unsigned int n=from; n<to;n++){
		CASE c = cases_.at(n);
		Case* ac = new Case(c.name, cb);
		for (unsigned int i = 0; i < c.features.size(); i++){
			FEATURE f = c.features.at(i);
			ac->addFeature(f.name, f.value);
		}
	}
}

int CaseBaseGenerator::getCaseNumber(){
	return cases_.size();
}

